/*******************************************************************************
 * Copyright (c) 2000, 2004 QNX Software Systems and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     QNX Software Systems - Initial API and implementation
 *******************************************************************************/
package org.tigcc;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.cdt.core.IAddressFactory;
import org.eclipse.cdt.core.IBinaryParser;
import org.eclipse.cdt.core.IBinaryParser.IBinaryFile;
import org.eclipse.cdt.core.IBinaryParser.ISymbol;
import org.eclipse.cdt.utils.Addr32;
import org.eclipse.cdt.utils.Addr32Factory;
import org.eclipse.cdt.utils.BinaryObjectAdapter;
import org.eclipse.cdt.utils.Symbol;
import org.eclipse.core.runtime.IPath;

/**
 */
public class Ti68kBinaryObject extends BinaryObjectAdapter {

	BinaryObjectInfo info;
	IAddressFactory addressFactory;
	ISymbol[] symbols;

	public Ti68kBinaryObject(IBinaryParser parser, IPath path) {
		super(parser, path, IBinaryFile.OBJECT);
	}
	
	public Ti68kBinaryObject(IBinaryParser parser, IPath p, int type) {
		super(parser, p, type);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.cdt.utils.BinaryObjectAdapter#getName()
	 */
	public String getName() {
		return super.getName();
	}	

	/* (non-Javadoc)
	 * @see org.eclipse.cdt.core.IBinaryParser.IBinaryFile#getContents()
	 */
	public InputStream getContents() throws IOException {
		return super.getContents();
	}
	
	/**
	 * @see org.eclipse.cdt.core.model.IBinaryParser.IBinaryObject#getSymbols()
	 */
	public ISymbol[] getSymbols() {
		if (hasChanged() || symbols == null) {
			loadAll();
		}
		return symbols;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.cdt.utils.BinaryObjectAdapter#getBinaryObjectInfo()
	 */
	protected BinaryObjectInfo getBinaryObjectInfo() {
		if (hasChanged() || info == null) {
			loadInfo();
		}
		return info;
	}

	protected Ti68k getTi68k() {
		return new Ti68k(getPath().toOSString());
	}

	protected void loadAll() {
		Ti68k ti68k = null;
		try {
			ti68k = getTi68k();
			loadInfo(ti68k);
			loadSymbols(ti68k);
		} finally {
			if (ti68k != null) {
				ti68k.dispose();
			}
		}
	}

	protected void loadInfo() {
		Ti68k ti68k = null;
		try {
			ti68k = getTi68k();
			loadInfo(ti68k);
		} finally {
			if (ti68k != null) {
				ti68k.dispose();
			}
		}
	}

	protected void loadInfo(Ti68k ti68k) {
		info = new BinaryObjectInfo();
		Ti68k.Attribute attribute = getTi68k().getAttributes();
		info.isLittleEndian = attribute.isLittleEndian();
		info.hasDebug = attribute.hasDebug();
		info.cpu = attribute.getCPU();
	}

	protected void loadSymbols(Ti68k ti68k) {
		ArrayList list = new ArrayList();
		loadSymbols(ti68k, list);
		symbols = (ISymbol[]) list.toArray(NO_SYMBOLS);
		Arrays.sort(symbols);
		list.clear();
	}
	protected void loadSymbols(Ti68k ti68k, List list) {
		Ti68k.Symbol[] ti68kSyms = ti68k.getSymbols();
		byte[] table = ti68k.getStringTable();
		addSymbols(ti68kSyms, table, list);
	}

	protected void addSymbols(Ti68k.Symbol[] ti68kSyms, byte[] table, List list) {
		for (int i = 0; i < ti68kSyms.length; i++) {
			if (ti68kSyms[i].isFunction() || ti68kSyms[i].isPointer() || ti68kSyms[i].isArray()) {
				String name = ti68kSyms[i].getName(table);
				if (name == null || name.trim().length() == 0 || !Character.isJavaIdentifierStart(name.charAt(0))) {
					continue;
				}
				int type = ti68kSyms[i].isFunction() ? ISymbol.FUNCTION : ISymbol.VARIABLE;
				list.add(new Symbol(this, name, type, new Addr32(ti68kSyms[i].n_value), 1));
			}
		}
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.cdt.utils.BinaryObjectAdapter#getAddressFactory()
	 */
	public IAddressFactory getAddressFactory() {
		if (addressFactory == null) {
			addressFactory = new Addr32Factory();
		}
		return addressFactory;
	}
}